@@ -21,7 +21,7 @@
 ;; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
-;; This module provides a siple input method
+;; This module provides a simple input method
 ;; under the server-side control via IIIM.
 
 ;;; Code:
@@ -30,7 +30,7 @@
 
 ;;; version.
 
-(defconst iiimcf-server-control-version "0.1 (Kamo)")
+(defconst iiimcf-server-control-version "0.2 (Takase)")
 
 ;;; Customizable options.
 
@@ -105,6 +105,7 @@ If each hostname has the form of \"HOSTNAME:N\", use N-th port."
 (defvar iiimcf-server-control-im-id nil)
 (defvar iiimcf-server-control-ic-id nil)
 (defvar iiimcf-server-control-dynamic-event-flow-p nil)
+(defvar iiimcf-server-control-input-method-filtered-events nil)
 (defvar iiimcf-server-control-editing-buffer nil
   "The buffer where a user is inputting currently.")
 (defvar iiimcf-server-control-preedit "")
@@ -137,6 +138,14 @@ If each hostname has the form of \"HOSTNAME:N\", use N-th port."
 (defvar iiimcf-server-control-lookup-choice-configuration nil)
 
 ;;; keymap
+
+(defvar iiimcf-server-control-minor-mode-keymap
+  (let ((map (make-keymap)))
+    (define-key map "\C-c\C-\\\C-l" 'iiimcf-server-control-change-language)
+    (define-key map "\C-c\C-\\\C-c" 'iiimcf-server-control-switch-ic-attribute)
+    (define-key map "\C-c\C-\\\C-s" 'iiimcf-server-control-change-iiim-server)
+    map))
+
 (defvar iiimcf-server-control-keymap
   (let ((map (make-keymap))
 	(i 0))
@@ -459,7 +468,7 @@ If each hostname has the form of \"HOSTNAME:N\", use N-th port."
     ;;(run-hooks 'iiimcf-server-control-activate-hook)
     (make-local-variable 'input-method-function)
     (setq input-method-function
-	  (function iiimcf-server-control-keyforward))))
+	  (function iiimcf-server-control-input-method-filter))))
 
 (defun iiimcf-server-control-exit-from-minibuffer ()
   (inactivate-input-method)
@@ -469,18 +478,23 @@ If each hostname has the form of \"HOSTNAME:N\", use N-th port."
 		    iiimcf-server-control-exit-from-minibuffer))))
 
 (defun iiimcf-server-control-dispatch-event ()
-  (let* ((input-method-function nil)
-	 (keyseq (read-key-sequence
-		  nil nil nil t))
-	 (cmd (key-binding keyseq t)))
-    (if (and (commandp cmd)
-	     (not (eq cmd 'iiimcf-server-control-keyforward)))
-	(progn
-	  (setq last-command-event (aref keyseq
-					 (1- (length keyseq)))
-		last-command this-command
-		this-command cmd)
-	  (call-interactively cmd)))))
+  (let ((input-method-function nil)
+	keyseq cmd)
+    (while (input-pending-p)
+      (setq keyseq (read-key-sequence nil nil nil t)
+	    cmd (key-binding keyseq t))
+      (if (and (commandp cmd)
+	       (not (eq cmd 'iiimcf-server-control-keyforward)))
+	  (progn
+	    (setq last-command-event (aref keyseq
+					   (1- (length keyseq)))
+		  last-command this-command
+		  this-command cmd)
+	    (call-interactively cmd))))))
+
+(defun iiimcf-server-control-input-method-filter (&optional ev)
+  (let (iiimcf-server-control-input-method-filtered-events)
+    (iiimcf-server-control-keyforward ev)))
 
 (defun iiimcf-server-control-keyforward (&optional ev)
   (interactive)
@@ -522,7 +536,7 @@ If each hostname has the form of \"HOSTNAME:N\", use N-th port."
 	     unread-command-events
 	     (list event)))
       (iiimcf-server-control-dispatch-event))
-    nil))
+    iiimcf-server-control-input-method-filtered-events))
 
 (defun iiimcf-server-control-trigger-notify (mes)
   (let ((flag (nth 4 mes)))
@@ -730,6 +744,19 @@ If each hostname has the form of \"HOSTNAME:N\", use N-th port."
 	  ;; text from modifications of the face text-property by font-lock.
 	  keyseq cmd)
       (while (> (length iiimcf-server-control-preedit) 0)
+	(if iiimcf-server-control-input-method-filtered-events
+	    (save-excursion
+	      (set-buffer
+	       (marker-buffer
+		iiimcf-server-control-preedit-marker))
+	      (goto-char
+	       (marker-position
+		iiimcf-server-control-preedit-marker))
+	      (setq unread-command-events
+		    (nconc unread-command-events
+			   iiimcf-server-control-input-method-filtered-events))
+	      (iiimcf-server-control-dispatch-event)
+	      (setq iiimcf-server-control-input-method-filtered-events nil)))
 	(let ((overriding-terminal-local-map
 	       iiimcf-server-control-keymap)
 	      (input-method-function nil))
@@ -922,12 +949,20 @@ If each hostname has the form of \"HOSTNAME:N\", use N-th port."
 		       str
 		     (string-make-unibyte str)))
 		  generated-events))
-    (setq unread-command-events
-	  (nconc
-	   (if (and input-method-exit-on-first-char generated-events)
-	       (list (car generated-events))
-	     generated-events)
-	   unread-command-events))
+    (if (eq input-method-function
+	    (function iiimcf-server-control-input-method-filter))
+	(setq iiimcf-server-control-input-method-filtered-events
+	      (nconc
+	       (if (and input-method-exit-on-first-char generated-events)
+		   (list (car generated-events))
+		 generated-events)
+	       iiimcf-server-control-input-method-filtered-events))
+      (setq unread-command-events
+	    (nconc
+	     (if (and input-method-exit-on-first-char generated-events)
+		 (list (car generated-events))
+	       generated-events)
+	     unread-command-events)))
     ;; (insert str)
     (if pt (set-marker
 	    iiimcf-server-control-preedit-marker
